"use strict";
var MetadataStorage_1 = require("./metadata/MetadataStorage");
var ConstructorUtils = (function () {
    function ConstructorUtils() {
    }
    // -------------------------------------------------------------------------
    // Adder Methods
    // -------------------------------------------------------------------------
    ConstructorUtils.prototype.constructorToPlain = function (object, options) {
        var _this = this;
        if (object instanceof Array) {
            return object.map(function (item) {
                if (item instanceof Object) {
                    return _this.convert(item.constructor, item, "constructorToPlain");
                }
                else {
                    return item;
                }
            });
        }
        else {
            var cls = object.constructor;
            return this.convert(cls, object, "constructorToPlain", options);
        }
    };
    ConstructorUtils.prototype.plainToConstructorFromObject = function (object, json, options) {
        return this.convert(object, json, "plainToConstructor", options);
    };
    ConstructorUtils.prototype.plainToConstructor = function (cls, json, options) {
        return this.convert(cls, json, "plainToConstructor", options);
    };
    ConstructorUtils.prototype.plainToConstructorArray = function (cls, json, options) {
        var _this = this;
        if (!(json instanceof Array))
            throw new Error("Cannot convert given data to array of constructors. Given object is not an array.");
        return json.map(function (item) { return _this.plainToConstructor(cls, item, options); });
    };
    // -------------------------------------------------------------------------
    // Adder Methods
    // -------------------------------------------------------------------------
    ConstructorUtils.prototype.getMethods = function (obj) {
        var ret = Object.getOwnPropertyNames(obj);
        while (true) {
            obj = Object.getPrototypeOf(obj);
            var arr = void 0;
            try {
                arr = Object.getOwnPropertyNames(obj);
            }
            catch (e) {
                break;
            }
            for (var i = 0; i < arr.length; i++) {
                if (ret.indexOf(arr[i]) == -1)
                    ret.push(arr[i]);
            }
        }
        return ret;
    };
    ConstructorUtils.prototype.convert = function (cls, object, operationType, options) {
        var _this = this;
        if (object === null || object === undefined)
            return object;
        var newObject = {};
        if (operationType === "plainToConstructor") {
            if (cls instanceof Function) {
                newObject = new cls();
            }
            else {
                newObject = cls;
                cls = newObject.constructor;
            }
        }
        var keys = Object.keys(object);
        if (operationType === "constructorToPlain") {
            keys = keys.concat(this.getMethods(cls.prototype)).filter(function (key) { return [
                'constructor',
                'toString',
                'toLocaleString',
                'valueOf',
                'hasOwnProperty',
                'isPrototypeOf',
                'propertyIsEnumerable',
                '__defineGetter__',
                '__lookupGetter__',
                '__defineSetter__',
                '__lookupSetter__',
                '__proto__'
            ].indexOf(key) === -1; });
        }
        var _loop_1 = function(key) {
            if (this_1.isSkipped(cls, key, operationType))
                return "continue";
            if (typeof object[key] !== "function" && object[key] !== undefined) {
                if (options && options.skipStartedWith &&
                    key.substr(0, options.skipStartedWith.length) === options.skipStartedWith)
                    return "continue";
                var type_1 = this_1.getType(cls, key, newObject);
                if (!type_1 && operationType === "constructorToPlain") {
                    if (!(object[key] instanceof Array) && object[key] instanceof Object && object[key].constructor) {
                        type_1 = object[key].constructor;
                    }
                }
                if (object[key] instanceof Array) {
                    // if (object[key].length > 0 && !type && operationType === "plainToConstructor")
                    //     throw new TypeMissingError(cls, key);
                    var reflectedType = this_1.getReflectedType(cls, key);
                    newObject[key] = reflectedType && operationType === "plainToConstructor" ? new reflectedType() : [];
                    if (object[key].length > 0) {
                        object[key].forEach(function (arrayItem) {
                            if (!type_1 && operationType === "constructorToPlain" && arrayItem instanceof Object && arrayItem.constructor) {
                                type_1 = arrayItem.constructor;
                            }
                            if (type_1) {
                                newObject[key].push(_this.convert(type_1, arrayItem, operationType));
                            }
                            else {
                                newObject[key].push(arrayItem);
                            }
                        });
                    } /*else {
                        object[key].forEach((arrayItem: any) => {
                            newObject[key].push(arrayItem);
                        });
                    }*/
                }
                else if (object[key] instanceof Date && type_1 === Date && operationType === "constructorToPlain") {
                    newObject[key] = object[key].toISOString();
                }
                else if (object[key] instanceof Object && typeof object[key] === "object" && type_1) {
                    // if (!type && operationType === "plainToConstructor")
                    //     throw new TypeMissingError(cls, key);
                    newObject[key] = this_1.convert(type_1, object[key], operationType);
                }
                else {
                    if (type_1 === Date) {
                        newObject[key] = new Date(object[key]);
                    }
                    else if (type_1 === String) {
                        newObject[key] = String(object[key]);
                    }
                    else if (type_1 === Number) {
                        newObject[key] = Number(object[key]);
                    }
                    else if (type_1 === Boolean) {
                        newObject[key] = Boolean(object[key]);
                    }
                    else {
                        newObject[key] = object[key];
                    }
                }
            }
        };
        var this_1 = this;
        for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
            var key = keys_1[_i];
            var state_1 = _loop_1(key);
            if (state_1 === "continue") continue;
        }
        return newObject;
    };
    ConstructorUtils.prototype.isSkipped = function (target, propertyName, operationType) {
        if (!target)
            return undefined;
        var meta = MetadataStorage_1.defaultMetadataStorage.findSkipMetadata(target, propertyName);
        return operationType === "constructorToPlain" ? meta && meta.isConstructorToPlain : meta && meta.isPlainToConstructor;
    };
    ConstructorUtils.prototype.getType = function (target, propertyName, object) {
        if (!target)
            return undefined;
        var meta = MetadataStorage_1.defaultMetadataStorage.findTypeMetadata(target, propertyName);
        return meta ? meta.typeFunction(object) : undefined;
    };
    ConstructorUtils.prototype.getReflectedType = function (target, propertyName) {
        if (!target)
            return undefined;
        var meta = MetadataStorage_1.defaultMetadataStorage.findTypeMetadata(target, propertyName);
        return meta ? meta.reflectedType : undefined;
    };
    return ConstructorUtils;
}());
exports.ConstructorUtils = ConstructorUtils;
