"use strict";
var MetadataStorage_1 = require("./metadata/MetadataStorage");
var TypeMetadata_1 = require("./metadata/TypeMetadata");
var SkipMetadata_1 = require("./metadata/SkipMetadata");
/**
 * Specifies a type of the property.
 */
function Type(typeFunction) {
    return function (target, key) {
        var type = Reflect.getMetadata("design:type", target, key);
        var isArray = type && typeof type === "string" ? type.toLowerCase() === "array" : false;
        var metadata = new TypeMetadata_1.TypeMetadata(target.constructor, key, type, typeFunction, isArray);
        MetadataStorage_1.defaultMetadataStorage.addTypeMetadata(metadata);
    };
}
exports.Type = Type;
/**
 * Specifies a type of the property. Property must be an array.
 */
function ArrayType(typeFunction) {
    return function (target, key) {
        var type = Reflect.getMetadata("design:type", target, key);
        var metadata = new TypeMetadata_1.TypeMetadata(target.constructor, key, type, typeFunction, true);
        MetadataStorage_1.defaultMetadataStorage.addTypeMetadata(metadata);
    };
}
exports.ArrayType = ArrayType;
/**
 * Marks property as skipped on the process of transformation. By default it skips the property for both
 * constructorToPlain and plainToConstructor transformations, however you can specify on which of transformation types
 * you want to skip this property.
 */
function Skip(options) {
    return function (target, key) {
        var metadata = new SkipMetadata_1.SkipMetadata(target.constructor, key, options);
        MetadataStorage_1.defaultMetadataStorage.addSkipMetadata(metadata);
    };
}
exports.Skip = Skip;
